#!/usr/bin/env python
#
# -------------------------------------------------------------------------------------
#
# Copyright (c) 2016 KGH
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification, 
# are permitted provided that the following conditions are met:
#
#    - Redistributions of source code must retain the above copyright notice, this 
#    list of conditions and the following disclaimer.
#    - Redistributions in binary form must reproduce the above copyright notice, 
#    this list of conditions and the following disclaimer in the documentation and/or 
#    other materials provided with the distribution.
#    - Neither the name of the author nor the names of its contributors may be used 
#    to endorse or promote products derived from this software without specific prior 
#    written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY 
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
# SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
# TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
# DAMAGE.
#
# -------------------------------------------------------------------------------------
#
# This file is a basic Python plug-in for GIMP.
#
# It can be executed by selecting the menu option: 'Filters>Custom Font Labels'
#

from gimpfu import *
from array import array
from gtk import gdk
import math
import random

w = gdk.screen_width()
h = gdk.screen_height()

# Some global colours we may need to use

black = gimpcolor.RGB(0, 0, 0)
white = gimpcolor.RGB(255, 255, 255)
darkgrey = gimpcolor.RGB(50, 50, 50)
midgrey = gimpcolor.RGB(125, 125, 125)
lightgrey = gimpcolor.RGB(170, 170, 170)
verylightgrey = gimpcolor.RGB(222, 222, 222)
red = gimpcolor.RGB(220,10,10)	


def script_fu_gt_bevel(img, in_layer, bump_curve, bevel_depth, bump_depth, bump_azimuth, bump_elevation, bump_ambient, stretch_contrast, bump_invert, keep_layers):

#
#Declare Variables
#

    height = 0 				# Drawable Height
    width = 0				# Drawable Width
    bump_layer = -1 		# Bumpmap Layer
    bevel_layer = -1		# Bevel Layer
    saved_selection = -1	# Saved Selection
    saved_flag = False		# Saved Selection Flag 
    in_layer_pos = -1		# Current Selected Layer Position
    loop_counter = 1		# Loop Counter
    bump_color = 0			# Bumpmap Fill Color
    layer_offsets = 0		# Layer Offset From Image 
	
    # Save Context
    gimp.context_push()
    # Start Undo Group
    pdb.gimp_image_undo_group_start(img)
	
   
#Selection Check
	 
    if pdb.gimp_selection_is_empty(img): # Check For A Selection 
        pdb.gimp_image_select_item(img, CHANNEL_OP_REPLACE, in_layer) # Select Alpha to Logo
    else:
        saved_selection = pdb.gimp_selection_save(img) # Save Selection
        saved_flag = True # Set Saved Selection Flag
		
#Determine Layer Size

    width = pdb.gimp_drawable_width(in_layer)				# Active Layer Width
    height = pdb.gimp_drawable_height(in_layer) 			# Active Layer Height
    layer_offsets = pdb.gimp_drawable_offsets(in_layer) 	# Active Layer Offsets
	
# Create New Bevel Layer

    bevel_layer = pdb.gimp_layer_new(img, width, height, RGBA_IMAGE, "Bevel", 100, LAYER_MODE_NORMAL_LEGACY)
    in_layer_pos = pdb.gimp_image_get_item_position(img, in_layer)		# Get Incomimg Layer Position 
    pdb.gimp_image_insert_layer(img, bevel_layer, None, in_layer_pos)		# Insert Bevel layer
    pdb.gimp_layer_set_offsets(bevel_layer, layer_offsets[0], layer_offsets[1])		# Compensate For An Offset Layer
    pdb.gimp_context_set_background(gimpcolor.RGB(127,127,127)) #  Set BG Color to Gray (Python-Fu format for colours passed here)
    pdb.gimp_edit_bucket_fill(bevel_layer, BUCKET_FILL_BG, LAYER_MODE_NORMAL_LEGACY, 100, 0, False, 0, 0) # Fill Bevel Layer With Gray 
	
# Create New Bumpmap Layer

    bump_layer = pdb.gimp_layer_new(img, width, height, RGBA_IMAGE, "Bumpmap", 100, LAYER_MODE_NORMAL_LEGACY)
    in_layer_pos = pdb.gimp_image_get_item_position(img, in_layer)		# Get Incomimg Layer Position 
    pdb.gimp_image_insert_layer(img, bump_layer, None, in_layer_pos + 1)		# Insert Bump layer (Python-Fu requires None rather than 0 as the third parameter here)
    pdb.gimp_layer_set_offsets(bump_layer, layer_offsets[0], layer_offsets[1])		# Compensate For An Offset Layer
	
    while loop_counter < bevel_depth:
        bump_color = int((loop_counter * 255) / bevel_depth) # Bump Fill Color (must be an integer in Python-Fu)
        pdb.gimp_context_set_background(gimpcolor.RGB(bump_color, bump_color, bump_color)) #  Set Fill Color to BG

        # If Selection Empty There Is No Space To Fill, So Don't
        if pdb.gimp_selection_is_empty(img) == False:
            pdb.gimp_edit_bucket_fill(bump_layer, BUCKET_FILL_BG, LAYER_MODE_NORMAL_LEGACY, 100, 0, False, 0, 0) # Fill Slice
			
        pdb.gimp_selection_shrink(img, 1) # Shrink the selection for next sliver
        loop_counter += 1 # Increment Loop Counter
    # loop

# Bumpmap Incoming Layer

    if stretch_contrast:
        pdb.plug_in_autostretch_hsv(img, bump_layer) # Stretch Contrast 
		
    pdb.gimp_selection_none(img) # Clear the selection
	
    pdb.plug_in_bump_map(img, bevel_layer, bump_layer, bump_azimuth, bump_elevation, bump_depth, 0, 0, 0, bump_ambient, True, bump_invert, bump_curve)
	
    pdb.gimp_item_set_visible(bump_layer, False) # Set Bumpmap To Not Visible
    pdb.plug_in_colortoalpha(img, bevel_layer, gimpcolor.RGB(127,127,127)) # Remove BG Gray Leaving Only Bevel
    pdb.gimp_image_set_active_layer(img, bevel_layer) # Make Bevel Layer Active
	
# Remove Bumpmap

    if keep_layers == False:
        pdb.gimp_image_merge_down(img, bevel_layer, EXPAND_AS_NECESSARY) # Merge Down Bevel Layer
        pdb.gimp_image_remove_layer(img, bump_layer) # Remove The Bumpmap Layer
		
# Check For A Saved Selection

    if saved_flag:
        pdb.gimp_image_select_item(img, CHANNEL_OP_REPLACE, saved_selection)
	
	
    # Set end undo point
    pdb.gimp_image_undo_group_end(img)

    # Restore the GIMP's settings
    gimp.context_pop()
	
# Update Display

    pdb.gimp_displays_flush()
	


register(
    "python_fu_GT_Bevel_v100",
    " Bevel The Selected Layer",
    " Bevel The Selected Layer",
    "GnuTux - http://gimpchat.com",
    "GnuTux - GPLv3",
    "2018",
    "<Image>/Filters/Decor/py_fu_GT_Bevel_v100...",
    "RGB*",
    [
        (PF_OPTION, "slope_option", "Bevel Slope", 0, ("Flat", "Curved", "Sinus")),
        (PF_SPINNER, "width_size", "Bevel Width", 10, (0, 200, 1)),
        (PF_SPINNER, "height_size", "Bevel Height", 15, (1, 65, 1)),
        (PF_SPINNER, "azimuth", "Light Direction", 135, (0, 360, 1)),
        (PF_SPINNER, "distance", "Light Distance", 45.0, (0.5, 90.0, 0.5)),
        (PF_SPINNER, "ambient", "Ambient Light", 0.5, (0.0, 1.0, 0.01)),
        (PF_TOGGLE, "contrast", "Stretch Contrast", 0),
        (PF_TOGGLE, "invert", "Invert Bumpmap", 0),
        (PF_TOGGLE, "keep_layers", "Keep Layers", 0),
    ],
    [],
    script_fu_gt_bevel)

main()

	

